from datetime import datetime

import logging
import os

logger = logging.getLogger(__name__)


def setup_outputdir(path, warn_if_exist=True, create_dir=True, path_suffix=None):
    if path_suffix is None:
        path_suffix = ''
    if path_suffix and path_suffix[-1] == os.path.sep:
        path_suffix = path_suffix[:-1]
    if path is not None:
        path = f'{path}{path_suffix}'
    if path is None:
        utcnow = datetime.utcnow()
        timestamp = utcnow.strftime("%Y%m%d_%H%M%S")
        path = f"AutogluonModels/ag-{timestamp}{path_suffix}{os.path.sep}"
        for i in range(1, 1000):
            try:
                if create_dir:
                    os.makedirs(path, exist_ok=False)
                    break
                else:
                    if os.path.isdir(path):
                        raise FileExistsError
                    break
            except FileExistsError as e:
                path = f"AutogluonModels/ag-{timestamp}-{i:03d}{path_suffix}{os.path.sep}"
        else:
            raise RuntimeError("more than 1000 jobs launched in the same second")
        logger.log(25, f'No path specified. Models will be saved in: "{path}"')
    elif warn_if_exist:
        try:
            if create_dir:
                os.makedirs(path, exist_ok=False)
            elif os.path.isdir(path):
                raise FileExistsError
        except FileExistsError as e:
            logger.warning(f'Warning: path already exists! This predictor may overwrite an existing predictor! path="{path}"')
    path = os.path.expanduser(path)  # replace ~ with absolute path if it exists
    if path[-1] != os.path.sep:
        path = path + os.path.sep
    return path



import boto3, os, pathlib, logging

from . import load_pd
from ..utils import s3_utils

logger = logging.getLogger(__name__)

def list_bucket_s3(bucket):
    logger.log(15, 'Listing s3 bucket: '+str(bucket))

    s3bucket = boto3.resource('s3')
    my_bucket = s3bucket.Bucket(bucket)
    files = []
    for object in my_bucket.objects.all():
        files.append(object.key)
        logger.log(15, str(object.key))
    return files


def download(input_bucket, input_prefix, local_path):
    directory = os.path.dirname(local_path)
    pathlib.Path(directory).mkdir(parents=True, exist_ok=True)

    s3 = boto3.resource('s3')
    s3.Bucket(input_bucket).download_file(input_prefix, local_path)


def list_bucket_prefix_s3(bucket, prefix):
    return list_bucket_prefix_suffix_s3(bucket=bucket, prefix=prefix)


def list_bucket_prefix_suffix_s3(bucket, prefix, suffix=None, banned_suffixes=None):
    if banned_suffixes is None:
        banned_suffixes = []
    s3 = boto3.resource('s3')
    my_bucket = s3.Bucket(bucket)
    prefix = prefix

    files = []
    for object_summary in my_bucket.objects.filter(Prefix=prefix):
        suffix_full = object_summary.key.split(prefix, 1)[1]
        is_banned = False
        for banned_suffix in banned_suffixes:
            if banned_suffix in suffix_full:
                is_banned = True
        if (not is_banned) and ((suffix is None) or (suffix in suffix_full)):
            files.append(object_summary.key)

    return files


def list_bucket_prefix_suffix_contains_s3(bucket, prefix, suffix=None, banned_suffixes=None, contains=None):
    if banned_suffixes is None:
        banned_suffixes = []
    s3 = boto3.resource('s3')
    my_bucket = s3.Bucket(bucket)
    prefix = prefix

    files = []
    for object_summary in my_bucket.objects.filter(Prefix=prefix):
        suffix_full = object_summary.key.split(prefix, 1)[1]
        is_banned = False
        for banned_suffix in banned_suffixes:
            if banned_suffix in suffix_full:
                is_banned = True
        if (not is_banned) and ((suffix is None) or (suffix in suffix_full)) and (contains is None or contains in suffix_full):
            files.append(object_summary.key)

    return files


def load_multipart_s3(bucket, prefix, columns_to_keep=None, dtype=None, sample_count=None):
    files = list_bucket_prefix_s3(bucket, prefix)
    files_cleaned = [file for file in files if prefix + '/part-' in file]
    paths_full = [s3_utils.s3_bucket_prefix_to_path(bucket=bucket, prefix=file, version='s3') for file in files_cleaned]
    if sample_count is not None:
        logger.log(15, 'Taking sample of '+str(sample_count)+' of '+str(len(paths_full))+' s3 files to load')
        paths_full = paths_full[:sample_count]

    df = load_pd.load(path=paths_full, columns_to_keep=columns_to_keep, dtype=dtype)
    return df
